#!/bin/sh
set -e
trap Cleanup EXIT


################################################################################
# This function prints All parameters sent to it.
#
# @param $* - Variable number of strings.
################################################################################
Print ()
{
  printf %s\\n "$*"
}


################################################################################
# This function removes all events from a config file.
#
# @param $1 - The name of the config file to remove the events from.
################################################################################
RemoveEvents ()
{
  RE_configFileName=$1
  sed -i '/<event /,/<\/event>/ d' $RE_configFileName
}


################################################################################
# This function finds the nice name of a profile in the config file.
#
# @param $1 - The name of the config file.
# @param $2 - The id of the profile, an integer.
# @param $3 - The number of the area.
#
# @return The nice name of the profile, a string.
################################################################################
FindProfileNiceName ()
{
  FPNN_configFileName=$1
  FPNN_profileId=$2


  FPNN_profileLine=$(grep 'Profile'$FPNN_profileId'_NiceName' $FPNN_configFileName || Print '')
  FPNN_profileLineAfterValue=$(Print $FPNN_profileLine | sed 's/.*value=\"//')
  FPNN_profileNiceName=$(Print $FPNN_profileLineAfterValue | sed 's/\".*//')

  Print $FPNN_profileNiceName
}


################################################################################
# This function finds if a profile in the config file should write its events.
#
# @param $1 - The name of the config file.
# @param $2 - The id of the profile, an integer.
#
# @return The value if the profile is enabled, a string [1 / 0].
################################################################################
IsAreaEnabled ()
{
  IAE_configFileName=$1
  IAE_profileId=$2
  IAE_areaId=$3

  IAE_SearchString='Profile'$IAE_profileId'_IncludeArea'$IAE_areaId

  IAE_Line=$(grep "$IAE_SearchString" $IAE_configFileName || Print '')

  if [ "$IAE_Line" != "" ]
  then
    Print '1'
  else
    Print '0'
  fi
}

################################################################################
# This function finds if a profile in the config file should write its events.
#
# @param $1 - The name of the config file.
# @param $2 - The id of the profile, an integer.
#
# @return The value if the profile is enabled, a string [1 / 0].
################################################################################
IsProfileEnabled ()
{
  IPE_configFileName=$1
  IPE_profileId=$2

  IPE_profileLine=$(grep 'Profile'$IPE_profileId'_IsEnabled' $IPE_configFileName || Print '')
  IPE_profileLineAfterValue=$(Print $IPE_profileLine | sed 's/.*value=\"//')
  IPE_profileNiceName=$(Print $IPE_profileLineAfterValue | sed 's/\".*//')

  Print $IPE_profileNiceName
}


################################################################################
# This function finds a line number to write an event on.
#
# @param $1 - The name of the config file.
#
# @return The line number to write an event on, an integer.
################################################################################
FindNewEventLineNbr ()
{
  FNELN_configFileName=$1

  FNELN_eventEndLinesWithLineNbr=$(grep -n '</event>' $FNELN_configFileName || Print '')
  FNELN_eventEndLineNbrs=$(Print $FNELN_eventEndLinesWithLineNbr | sed 's/:.*<\/event>//g')
  FNELN_lastEventEndLineNbr=$(Print $FNELN_eventEndLineNbrs | sed 's/[0-9]* //')

  if [ "$FNELN_lastEventEndLineNbr" = '' ]; then
    FNELN_eventsLineWithLineNbr=$(grep -n '<events>' $FNELN_configFileName)
    FNELN_eventsLineNbr=$(Print $FNELN_eventsLineWithLineNbr | sed 's/: <events>//')
    Print $FNELN_eventsLineNbr
  else
    Print $FNELN_lastEventEndLineNbr
  fi
}


################################################################################
# This function writes an event to a config file.
#
# @param $1 - The name of the config file.
# @param $2 - The line number in the config file to write on.
# @param $3 - The nice name of the event.
# @param $4 - The channel number.
# @param $5 - The include area number.
# @param $6 - The profile number.
# @param $7 - The total number of channels.
################################################################################
WriteEvent ()
{
  WE_configFileName=$1
  WE_startLineNbr=$2
  WE_eventNiceName=$3
  WE_channelNbr=$4
  WE_profileNbr=$5
  WE_totalNbrChannels=$6

  WE_newLine=''
  WE_pad1=''
  WE_pad2=''
  WE_attrProfileStr='<attr key="profile" nicename="Profile ID" tag="source" value="'$WE_profileNbr'"/>'
  WE_attrPassedFenceStr='<attr key="type" nicename="Passed Fence" value="passed"/>'
  WE_attrFenceLineStr='<attr key="line" nicename="Fence Line" tag="data"/>'
  WE_attrPassedObjeIDStr='<attr key="object" nicename="Passed object id" tag="data"/>'
  WE_eventEndStr='</event>'
  WE_eventStr='<event name="fenceguard_video_profile_'$WE_profileNbr'_camera_'$WE_channelNbr'"'

  WE_eventNiceNameStr=' nicename="Fence Guard: '$WE_eventNiceName'">'
  #WE_eventNiceNameStr=WE_eventNiceNameStr' (Area '$WE_areaNbr')'    #Uncomment this line if the ACAP supports multiple include areas.
  if [ $WE_totalNbrChannels -gt 1 ]; then
    WE_eventNiceNameStr=' nicename="Fence Guard (Video '$WE_channelNbr'): '$WE_eventNiceName'">'
    #WE_eventNiceNameStr=WE_eventNiceNameStr' (Area '$WE_areaNbr')'    #Uncomment this line if the ACAP supports multiple include areas.
  fi

  WE_eventStr="${WE_eventStr}""${WE_eventNiceNameStr}"

  WE_totalEventStr="${WE_pad1}""${WE_eventStr}""${WE_newLine}"\
"${WE_pad2}""${WE_attrProfileStr}""${WE_newLine}"\
"${WE_pad2}""${WE_attrPassedFenceStr}""${WE_newLine}"\
"${WE_pad2}""${WE_attrFenceLineStr}""${WE_newLine}"\
"${WE_pad2}""${WE_attrPassedObjeIDStr}""${WE_newLine}"\
"${WE_pad1}""${WE_eventEndStr}"

  sed -i "${WE_startLineNbr} a\ $WE_totalEventStr" $WE_configFileName
}


################################################################################
# This function writes all needed events to a config file.
#
# @param $1 - The name of the config file.
# @param $2 - The maximum number of profiles.
# @param $3 - The maximum number of include areas per profile.
# @param $4 - The total number of channels.
################################################################################
WriteAllEvents ()
{
  WAE_configFileName=$1
  WAE_maxNbrProfiles=$2
  WAE_totalNbrChannels=$3

  WAE_channelNbr=$(GetChannelNbr $WAE_configFileName)

  WAE_profileNbr=0
  while [ $WAE_profileNbr -lt $WAE_maxNbrProfiles ]; do
    WAE_areaNbr=0
    WAE_profileEnabled=$(IsProfileEnabled $WAE_configFileName $WAE_profileNbr)
    if [ "$WAE_profileEnabled" = "1" ]; then
      WAE_profileNiceName=$(FindProfileNiceName $WAE_configFileName $WAE_profileNbr)
      WAE_eventLineNbr=$(FindNewEventLineNbr $WAE_configFileName)

      WAE_eventLineNbr=$(FindNewEventLineNbr $WAE_configFileName)
      WriteEvent $WAE_configFileName $WAE_eventLineNbr "$WAE_profileNiceName" $WAE_channelNbr $WAE_profileNbr $WAE_totalNbrChannels

    fi
    WAE_profileNbr=$(($WAE_profileNbr + 1))
done
}


################################################################################
# This function checks if a file has multiple lines.
#
# @param $1 - The name of the file.
#
# @return 'true' if the file has multiple lines, otherwise 'false', a string.
################################################################################
DoesFileHaveMultipleLines ()
{
  DFHML_fileName=$1

  DFHML_nbrLinesInFile=$(wc -l < $DFHML_fileName)
  if [ $DFHML_nbrLinesInFile -gt 10 ]; then #1 should be enough but some extra new lines have been added to be on the safe side
    Print 'true'
  else
    Print 'false'
  fi
}

################################################################################
# This function inserts new line characters in a file
# every instance of the characters
# '></'
# are changed to
# '>
# </'.
#
# @param $1 - The name of the file.
################################################################################
InsertNewLineCharactersInFile ()
{
  INLCIF_fileName=$1

  sed -i 's/><\//>\n<\//g' $INLCIF_fileName
}


InsertNewLineCharactersInFile2 ()
{
  INLCIF2_fileName=$1

  sed -i 's/></>\n</g' $INLCIF2_fileName
}

################################################################################
# This function gets the number of channels that are present in a config file.
#
# @param $1 - The name of the config file.
#
# @return The number of channels, an integer.
################################################################################
GetNbrChannels ()
{
  GNC_configFileName=$1

  GNC_nbrChannels=$(grep -c "<ruleEngine " $GNC_configFileName)
  Print $GNC_nbrChannels
}


################################################################################
# This function gets the channel number from a config file.
#
# @param $1 - The name of the config file.
#
# @return The channel number.
################################################################################
GetChannelNbr ()
{
  GCN_configFile=$1

  GCN_channelLine=$(grep 'Channel' $GCN_configFile)
  GCN_frontRemoved=$(Print $GCN_channelLine | sed 's/.*value=\"//')
  GCN_channelNbr=$(Print $GCN_frontRemoved | sed 's/\".*//')

  Print $GCN_channelNbr
}


################################################################################
# This function copies a specified number of lines from one file to another.
#
# @param $1 - The source file name.
# @param $2 - The destination file name.
# @param $3 - The line number to start the copying from, the line is included.
# @param $4 - The line number to stop the copying from, the line is included.
################################################################################
CopyLinesFromFileToFile ()
{
  CLFFTF_sourceFile=$1
  CLFFTF_destinationFile=$2
  CLFFTF_startLineNbr=$3
  CLFFTF_endLineNbr=$4

  CLFFTF_nbrLinesInSourceFile=$(wc -l < $CLFFTF_sourceFile)
  CLFFTF_tailParam=$(($CLFFTF_nbrLinesInSourceFile - $CLFFTF_startLineNbr + 1))
  CLFFTF_headParam=$(($CLFFTF_endLineNbr - $CLFFTF_startLineNbr + 1))

  tail -$CLFFTF_tailParam $CLFFTF_sourceFile | head -$CLFFTF_headParam > $CLFFTF_destinationFile
}


################################################################################
# This function deletes lines from a file.
#
# @param $1 - The source file name.
# @param $2 - The line number to start the copying from, the line is included.
# @param $3 - The line number to stop the copying from, the line is included.
################################################################################
DeleteLinesInFile ()
{
  DLIF_configFileName=$1
  DLIF_startLineNbr=$2
  DLIF_endLineNbr=$3

  sed -i "$DLIF_startLineNbr,$DLIF_endLineNbr d" $DLIF_configFileName
}


################################################################################
# This function creates the name of a temporary file.
#
# @param $1 - The id of the file, an integer.
#
# @return The name of the temporary file, a string.
################################################################################
CreateTemporaryFileName ()
{
  CTFN_id=$1

  Print "${gTmpDir}""/""${CTFN_id}""${gTmpFileName}"
}


################################################################################
# This function creates temporary files, one file per channel in the config
# file. The files are used to manipulate the settings for one channel.
# The data moved to the temporary files are removed from the config file.
#
# @param $1 - The name of the config file.
# @param $2 - The number of channels.
################################################################################
CreateTemporaryFiles ()
{
  CTF_configFileName=$1
  CTF_nbrChannels=$2

  CTF_i=0
  while [ $CTF_i -lt $CTF_nbrChannels ]; do
    CTF_ruleEngineStartLinesWithLineNbr=$(grep -n '<ruleEngine ' $CTF_configFileName)
    CTF_startLineNbr=$(Print $CTF_ruleEngineStartLinesWithLineNbr | sed 's/:.*//')
    CTF_ruleEngineEndLinesWithLineNbr=$(grep -n '</ruleEngine>' $CTF_configFileName)
    CTF_endLineNbr=$(Print $CTF_ruleEngineEndLinesWithLineNbr | sed 's/:.*//')

    CTF_tmpFileName=$(CreateTemporaryFileName $CTF_i)
    touch $CTF_tmpFileName

    CopyLinesFromFileToFile $CTF_configFileName $CTF_tmpFileName $CTF_startLineNbr $CTF_endLineNbr
    DeleteLinesInFile $CTF_configFileName $CTF_startLineNbr $CTF_endLineNbr

    CTF_i=$(($CTF_i + 1))
  done
}


################################################################################
# This function Parses the temporary files i.e. creates the events.
#
# @param $1 - The number of temporary files.
# @param $2 - The maximum number of profiles.
# @param $3 - The maximum number of include areas per profile.
################################################################################
ParseTemporaryFiles ()
{
  PTF_nbrTemporaryFiles=$1
  PTF_maxNbrProfiles=$2

  PTF_i=0
  while [ $PTF_i -lt $PTF_nbrTemporaryFiles ]; do
    PTF_tmpFileName=$(CreateTemporaryFileName $PTF_i)
    RemoveEvents $PTF_tmpFileName
    WriteAllEvents $PTF_tmpFileName $PTF_maxNbrProfiles $PTF_nbrTemporaryFiles

    PTF_i=$(($PTF_i + 1))
  done
}


################################################################################
# This function merges the temporary files to the config file.
#
# @param $1 - The name of the config file.
# @param $2 - The number of temporary files.
################################################################################
MergeTemporaryFilesToConfigFile ()
{
  MTFTCF_configFileName=$1
  MTFTCF_nbrTmpFiles=$2
  MTFTCF_correctOwnerFile=$3

  MTFTCF_tmpResultFile='/tmp/result_'$gTmpFileName
  if [ -f $MTFTCF_tmpResultFile ]; then
    rm $MTFTCF_tmpResultFile
  fi
  touch $MTFTCF_tmpResultFile

  MTFTCF_startRuleEnginesWithLineNbr=$(grep -n '<ruleEngines>' $MTFTCF_configFileName)
  MTFTCF_startRuleEnginesLineNbr=$(Print $MTFTCF_startRuleEnginesWithLineNbr | sed 's/: <ruleEngines>//')

  cat $MTFTCF_configFileName | head -$MTFTCF_startRuleEnginesLineNbr >> $MTFTCF_tmpResultFile

  MTFTCF_i=0
  while [ $MTFTCF_i -lt $MTFTCF_nbrTmpFiles ]; do
    MTFTCF_tmpFileName=$(CreateTemporaryFileName $MTFTCF_i)
    cat $MTFTCF_tmpFileName >> $MTFTCF_tmpResultFile
    MTFTCF_i=$(($MTFTCF_i + 1))
  done

  MTFTCF_endRuleEnginesWithLineNbr=$(grep -n '</ruleEngines>' $MTFTCF_configFileName)
  MTFTCF_endRuleEnginesLineNbr=$(Print $MTFTCF_endRuleEnginesWithLineNbr | sed 's/: <\/ruleEngines>//')
  MTFTCF_nbrLinesInFile=$(wc -l < $MTFTCF_configFileName)
  MTFTCF_tailParam=$(($MTFTCF_nbrLinesInFile - $MTFTCF_endRuleEnginesLineNbr + 1))

  cat $MTFTCF_configFileName | tail -$MTFTCF_tailParam >> $MTFTCF_tmpResultFile

  InsertNewLineCharactersInFile2 $MTFTCF_tmpResultFile

  echo '' >> $MTFTCF_tmpResultFile

  chown $(stat -c %U:%G "$MTFTCF_correctOwnerFile") $MTFTCF_tmpResultFile

  mv $MTFTCF_tmpResultFile $MTFTCF_configFileName
}


################################################################################
# This function deletes the temporary files.
#
# @param $1 - The number of temporary files.
################################################################################
DeleteTemporaryFiles ()
{
  DTF_nbrTmpFiles=$1

  DTF_i=0
  while [ $DTF_i -lt $DTF_nbrTmpFiles ]; do
    DTF_tmpFileName=$(CreateTemporaryFileName $DTF_i)
    rm $DTF_tmpFileName
    DTF_i=$(($DTF_i + 1))
  done
}


################################################################################
# This function removes all the temporary files.
################################################################################
Cleanup ()
{
  rm -rf $gTmpDir
}


################################################################################
#                                                                              #
#                                 START                                        #
#                                                                              #
################################################################################
gConfigFileName='/usr/local/packages/FenceGuard/FenceGuard.xml'
gTmpDir='/tmp/FenceGuardPostInstallDir'
gConfigFileCopyName=${gTmpDir}'/FenceGuardCopy.xml'
gTmpFileName='FenceGuardConfigTmp.xml'
gMaxNbrProfiles=5
gPackageFile='/usr/local/packages/FenceGuard/package.conf'

Print "Post install script Starting - " $(date)

Cleanup
mkdir $gTmpDir
echo ' ' >> $gConfigFileName
cp $gConfigFileName $gConfigFileCopyName


InsertNewLineCharactersInFile $gConfigFileCopyName
nbrChannels=$(GetNbrChannels $gConfigFileCopyName)

CreateTemporaryFiles $gConfigFileCopyName $nbrChannels
ParseTemporaryFiles $nbrChannels $gMaxNbrProfiles
MergeTemporaryFilesToConfigFile $gConfigFileName $nbrChannels $gPackageFile

Print "Post install script - Done"
